package de.firstdata.ipg.api.client.transaction.creditcard.object;

import de.firstdata.ipg.api.client.IPGApiClient;
import de.firstdata.ipg.api.client.transaction.Amount;
import de.firstdata.ipg.api.client.transaction.creditcard.CCPreAuthTransaction;
import de.firstdata.ipg.api.client.transaction.creditcard.CreditCard;
import de.firstdata.ipg.api.schema.EMVCardPresentRequest;
import de.firstdata.ipg.api.schema.Secure3DAuthenticationRequest;
import de.firstdata.ipg.api.schema.Secure3DVerificationRequest;
import de.firstdata.ipg.api.schema.SecurePlusVerifySmsCodeRequest;

/**
 * This class represents a <i>PreAuth</i> transaction and allows for setting its
 * optional attributes by calling the appropriate <code>setter</code> methods.
 * An instance of this class is created when calling
 * {@link IPGApiClient#createPreAuthTransaction(double, String, String, String)}
 * .
 *
 * @author Andreas Schmid
 */
public final class CCPreAuthTransactionObject extends CCTransactionObject implements CCPreAuthTransaction {

	/**
	 * creates a <i>PreAuth</i> transaction with only the transactionType set.
	 * This transaction is used for the authentication request call of the IVR
	 * Secure 3D transaction where the transaction data are loaded from the
	 * database.
	 */
	public CCPreAuthTransactionObject() {
		super("preAuth", null, null, null, null, null);
	}

	/**
	 * creates a <i>PreAuth</i> transaction with the minimum set of parameters -
	 * note that you can add optional fields by calling the appropriate
	 * <code>setter</code> methods on the returned
	 * {@link CCPreAuthTransactionObject} instance
	 *
	 */
	public CCPreAuthTransactionObject(final Amount amount, final CreditCard creditCard) {
		super("preAuth", amount, creditCard, null, null, null);
	}

	public CCPreAuthTransactionObject(final Amount amount, final String hostedDataID, final String hostedDataStoreID) {
		super("preAuth", amount, null, hostedDataID, hostedDataStoreID, null);
	}

	public CCPreAuthTransactionObject(final Amount amount, final EMVCardPresentRequest emvCardPresentRequest) {
		super("preAuth", amount, null, null, null, null);
		setEMVCardPresentRequest(emvCardPresentRequest);
	}

	@Override
	public void setSecure3DVerificationRequest(final Secure3DVerificationRequest secure3dVerificationRequest) {
		getSecure3DRequest().setSecure3DVerificationRequest(secure3dVerificationRequest);
	}

	@Override
	public void setSecure3DAuthenticationRequest(final Secure3DAuthenticationRequest secure3dAuthenticationRequest) {
		getSecure3DRequest().setSecure3DAuthenticationRequest(secure3dAuthenticationRequest);
	}

	@Override
	public void setVerifySmsCodeRequest(final SecurePlusVerifySmsCodeRequest securePlusVerifySmsCodeRequest) {
		getSecurePlusRequest().setSecurePlusVerifySmsCodeRequest(securePlusVerifySmsCodeRequest);
	}

	@Override
	public void setAuthenticateUpopTransaction(final Boolean value) {
		getUpop().setAuthenticateTransaction(value);
	}

}
