package de.firstdata.ipg.api.client.transaction.creditcard.object;

import java.net.URL;
import java.util.List;

import de.firstdata.ipg.api.client.transaction.Amount;
import de.firstdata.ipg.api.client.transaction.creditcard.CreditCard;
import de.firstdata.ipg.api.client.transaction.object.IPGApiTransactionObject;
import de.firstdata.ipg.api.schema.AirlineDetails;
import de.firstdata.ipg.api.schema.CarRental;
import de.firstdata.ipg.api.schema.CardEnrolledType;
import de.firstdata.ipg.api.schema.CreditCard3DSecure;
import de.firstdata.ipg.api.schema.CreditCardData;
import de.firstdata.ipg.api.schema.CreditCardTxType;
import de.firstdata.ipg.api.schema.DynamicPricing;
import de.firstdata.ipg.api.schema.HotelLodging;
import de.firstdata.ipg.api.schema.InquiryRateReference;
import de.firstdata.ipg.api.schema.PurchaseCard;
import de.firstdata.ipg.api.schema.RecurringType;
import de.firstdata.ipg.api.schema.SCAExemptionIndicators;
import de.firstdata.ipg.api.schema.Secure3DRequest;
import de.firstdata.ipg.api.schema.SecurePlusRequest;
import de.firstdata.ipg.api.schema.SettlementSplit;
import de.firstdata.ipg.api.schema.SplitShipment;
import de.firstdata.ipg.api.schema.SubMerchantSplit;
import de.firstdata.ipg.api.schema.Terminal;
import de.firstdata.ipg.api.schema.Upop;
import de.firstdata.ipg.api.schema.Wallet;
import de.firstdata.ipg.api.schema.WalletType;

/**
 * <code>IPGApiTransaction</code> is the abstract super class of any other
 * transaction class. It contains methods for setting optional fields which are
 * common to all transactions regardless of their type.
 *
 * @author Andreas Schmid
 *
 */
public abstract class CCTransactionObject extends IPGApiTransactionObject {

	protected CCTransactionObject(final String type, final Amount amount, final CreditCard creditCard, final String hostedDataID,
			final String hostedDataStoreID, final String orderId) {
		super(amount, hostedDataID, hostedDataStoreID, orderId);
		getCreditCardTxType().setType(type);
		if (creditCard == null) {
			return;
		}
		final CreditCardData cCD = new CreditCardData();
		cCD.setCardNumber(creditCard.cardNumber);
		cCD.setExpMonth(creditCard.expMonth);
		cCD.setExpYear(creditCard.expYear);
		cCD.setCardCodeValue(creditCard.cardCodeValue);
		cCD.setTrackData(creditCard.trackData);
		cCD.setBrand(creditCard.brand);
		setCreditCardData(cCD);
	}

	public void setAirlineDetails(final AirlineDetails airlineDetails) {
		getTransactionDetails().setAirlineDetails(airlineDetails);
	}

	public void setHotelLodging(final HotelLodging hotelLodging) {
		getTransactionDetails().setHotelLodging(hotelLodging);
	}

	public void setCarRental(final CarRental carRental) {
		getTransactionDetails().setCarRental(carRental);
	}

	public void setPurchaseCardDetails(final PurchaseCard purchaseCard) {
		getTransactionDetails().setPurchaseCard(purchaseCard);
	}

	@Override
	public String getType() {
		return getCreditCardTxType().getType();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.TransactionOptionsSetter#
	 * setStoreId (java.lang.String)
	 */
	@Override
	public void setStoreId(final String storeId) {
		getCreditCardTxType().setStoreId(storeId);
	}

	@Override
	public CreditCardTxType getCreditCardTxType() {
		if (this.creditCardTxType == null) {
			this.creditCardTxType = new CreditCardTxType();
		}
		return this.creditCardTxType;
	}

	public void setAuthenticationValue(final String value) {
		getCreditCard3DSecure().setAuthenticationValue(value);
	}

	public void setAuthenticate3dSecureTransaction(final Boolean value) {
		getCreditCard3DSecure().setAuthenticateTransaction(value);
	}

	public void setPayerAuthenticationResponse(final String value) {
		getCreditCard3DSecure().setPayerAuthenticationResponse(value);
	}

	public void setTerminalID(final String value) {
		getTerminal().setTerminalID(value);
	}

	public void setExternalMerchantID(final String value) {
		getTerminal().setMerchantID(value);
	}

	public void setMcc(final Integer value) {
		getTransactionDetails().setMcc(value);
	}

	public void setVerificationResponse(final String value) {
		getCreditCard3DSecure().setVerificationResponse(null == value ? null : CardEnrolledType.valueOf(value));
	}

	public void setXID(final String value) {
		getCreditCard3DSecure().setXID(value);
	}

	public void setDSRPECI(final String value) {
		getCreditCard3DSecure().setDSRPECI(value);
	}

	public void setOverride3DSCountryExclusion(final boolean override3DSCountryExclusion) {
		getCreditCard3DSecure().setOverride3DsCountryExclusion(Boolean.valueOf(override3DSCountryExclusion));
	}

	public void setSecure3DMethodNotificationStatus(final String secure3DMethodNotificationStatus) {
		getCreditCard3DSecure().setSecure3DMethodNotificationStatus(secure3DMethodNotificationStatus);
	}

	public void setTermUrl(final URL termUrl) {
		getCreditCard3DSecure().setTermUrl(termUrl.toString());
	}

	public void setThreeDSMethodNotificationURL(final URL threeDSMethodNotificationURL) {
		getCreditCard3DSecure().setThreeDSMethodNotificationURL(threeDSMethodNotificationURL.toString());
	}

	public void setThreeDSRequestorChallengeIndicator(final String threeDSRequestorChallengeIndicator) {
		getCreditCard3DSecure().setThreeDSRequestorChallengeIndicator(threeDSRequestorChallengeIndicator);
	}

	public void setThreeDSTransType(final String threeDSTransType) {
		getCreditCard3DSecure().setThreeDSTransType(threeDSTransType);
	}

	public void setSecure3D2AuthenticationResponse(final String authenticationResponse) {
		getCreditCard3DSecure().setSecure3D2AuthenticationResponse(authenticationResponse);
	}

	public void setSecure3D2TransactionStatus(final String transactionStatus) {
		getCreditCard3DSecure().setSecure3D2TransactionStatus(transactionStatus);
	}

	public void setSecure3DProtocolVersion(final String protocolVersion) {
		getCreditCard3DSecure().setSecure3DProtocolVersion(protocolVersion);
	}

	public void setAcsTransactionId(final String acsTransactionId) {
		getCreditCard3DSecure().setAcsTransactionId(acsTransactionId);
	}

	public void setThreeDSServerTransactionId(final String threeDSServerTransactionId) {
		getCreditCard3DSecure().setThreeDSServerTransactionId(threeDSServerTransactionId);
	}

	public void setDirectoryServerTransactionId(final String directoryServerTransactionId) {
		getCreditCard3DSecure().setDirectoryServerTransactionId(directoryServerTransactionId);
	}

	@Override
	public CreditCard3DSecure getCreditCard3DSecure() {
		if (this.creditCard3DSecure == null) {
			this.creditCard3DSecure = new CreditCard3DSecure();
		}
		return this.creditCard3DSecure;
	}

	protected Secure3DRequest getSecure3DRequest() {
		Secure3DRequest secure3DRequest = getCreditCard3DSecure().getSecure3DRequest();
		if (secure3DRequest == null) {
			secure3DRequest = new Secure3DRequest();
			getCreditCard3DSecure().setSecure3DRequest(secure3DRequest);
		}
		return secure3DRequest;
	}

	public void setDccApplied(final boolean dccApplied) {
		getInquiryRateReference().setDccApplied(dccApplied);
	}

	public void setInquiryRateId(final long inquiryRateId) {
		getInquiryRateReference().setInquiryRateId(inquiryRateId);
	}

	public void setForeignAmount(final Amount amount) {
		final DynamicPricing dynamicPricing = getDynamicPricing();
		dynamicPricing.setForeignCurrency(amount.currency);
		dynamicPricing.setForeignAmount(amount.total);
	}

	private DynamicPricing getDynamicPricing() {
		DynamicPricing dynamicPricing = getInquiryRateReference().getDynamicPricing();
		if (dynamicPricing == null) {
			dynamicPricing = new DynamicPricing();
			getInquiryRateReference().setDynamicPricing(dynamicPricing);
		}
		return dynamicPricing;
	}

	public void setSplitShipmentSequenceCount(final int splitShipmentSequenceCount) {
		getSplitShipment().setSequenceCount(Integer.valueOf(splitShipmentSequenceCount));
	}

	public void setFinalShipment(final boolean finalShipment) {
		getSplitShipment().setFinalShipment(finalShipment);
	}

	private SplitShipment getSplitShipment() {
		SplitShipment splitShipment = getTransactionDetails().getSplitShipment();
		if (splitShipment == null) {
			splitShipment = new SplitShipment();
			getTransactionDetails().setSplitShipment(splitShipment);
		}
		return splitShipment;
	}

	public void setWalletType(final WalletType value) {
		getWallet().setWalletType(value);
	}

	public void setWalletID(final String value) {
		getWallet().setWalletID(value);
	}

	@Override
	public Wallet getWallet() {
		if (this.wallet == null) {
			this.wallet = new Wallet();
		}
		return this.wallet;
	}

	private InquiryRateReference getInquiryRateReference() {
		InquiryRateReference ref = getTransactionDetails().getInquiryRateReference();
		if (ref == null) {
			ref = new InquiryRateReference();
			getTransactionDetails().setInquiryRateReference(ref);
		}
		return ref;
	}

	private Terminal getTerminal() {
		Terminal terminal = getTransactionDetails().getTerminal();
		if (terminal == null) {
			terminal = new Terminal();
			this.transactionDetails.setTerminal(terminal);
		}
		return terminal;
	}

	public void setInstallmentDelayMonths(final Integer installmentDelayMonths) {
		getPayment().setInstallmentDelayMonths(Integer.valueOf(installmentDelayMonths));
	}

	public void setNumberOfInstallments(final Integer numberOfInstallments) {
		getPayment().setNumberOfInstallments(Integer.valueOf(numberOfInstallments));
	}

	public void setInstallmentsInterest(final Boolean installmentsInterest) {
		if (installmentsInterest == null) {
			getPayment().setInstallmentsInterest(null);
		} else {
			getPayment().setInstallmentsInterest(installmentsInterest.booleanValue() ? "yes" : "no");
		}
	}

	public void setInstallmentPlanID(final String installmentPlanID) {
		getPayment().setInstallmentPlanID(installmentPlanID);
	}

	public void setStandIn(final boolean isStandin) {
		if (isStandin) {
			this.recurringType = RecurringType.STANDIN;
		} else {
			// remove recurring type only if that was standin
			if (RecurringType.STANDIN == this.recurringType) {
				this.recurringType = null;
			}
		}
	}

	@Override
	public Upop getUpop() {
		if (this.upop == null) {
			this.upop = new Upop();
		}
		return this.upop;
	}

	protected SecurePlusRequest getSecurePlusRequest() {
		SecurePlusRequest securePlusRequest = getUpop().getSecurePlusRequest();
		if (securePlusRequest == null) {
			securePlusRequest = new SecurePlusRequest();
			getUpop().setSecurePlusRequest(securePlusRequest);
		}
		return securePlusRequest;
	}

	public void setVCode(final String value) {
		getUpop().setVCode(value);
	}

	public void setResponseCode(final String value) {
		getUpop().setSendSmsResponseCode(value);
	}

	public void setActivateStatus(final String value) {
		getUpop().setActivateStatus(value);
	}

	public void setReferencedSchemeTransactionId(final String value) {
		getTransactionDetails().setReferencedSchemeTransactionId(value);
	}

	public void setSubMerchantSplit(final List<SubMerchantSplit> list) {
		final SettlementSplit settlementSplit = getSettlementSplit();
		final List<SubMerchantSplit> submerchantSplit = settlementSplit.getSubMerchantSplit();
		submerchantSplit.clear();
		submerchantSplit.addAll(list);
	}

	public SettlementSplit getSettlementSplit() {
		SettlementSplit settlementSplit = getTransactionDetails().getSettlementSplit();
		if (settlementSplit == null) {
			settlementSplit = new SettlementSplit();
			getTransactionDetails().setSettlementSplit(settlementSplit);
		}
		return settlementSplit;
	}

	public void setSCAExemptionIndicators(final SCAExemptionIndicators value) {
		getTransactionDetails().setSCAExemptionIndicators(value);
	}
}
